#!/usr/bin/env python
# Python 3.*

import sys
from termcolor import colored
        
class gcodeTerminalGenerator:
    def __init__(self): 
        if (len(sys.argv) <= 2):
            print(colored('Please provide a target folder and a name','yellow'))
            print()
            exit(0)
        else:
            self.createFile()
            self.header()
            self.app()
            self.footer()       
            
    def createFile(self):
        self.path = sys.argv[1] + '/' + sys.argv[2]
        self.fo = open(self.path, 'a')
        self.writer = gcodeWriter(self.fo)
        return self.writer
    
    def header(self):
        self.writer.addComment('Generated using python gcodeWriter')
        return self.writer

    def app(self):
        print(colored('Please define app(self)', 'yellow'))
        print()
        self.writer.addComment('Please define app(self)')
        exit(0)
        
    def footer(self):
        print(colored('%s successfully generated'%self.path, 'green'))
        print()
        return self.writer

class gcodeWriter:
    def __init__(self, fo):
        self.fo = fo
        self.coordinates = {'X': 0,
                            'Y': 0,
                            'Z': 0,
                            'E': 0}
    
    def addComment(self, comment):
        gcodeComment(comment).writeCommand(self.fo)
        return self
    
    def disableMotors(self):
        gcodeCommand('M','84').writeCommand(self.fo)
        return self
    
    def dwell(self, duration):
        gcodeCommandCollection([gcodeCommand('G', '4'), gcodeCommand('S', duration)]).writeCommand(self.fo)
        return self
    
    def fanOff(self):
        gcodeCommand('M','107').writeCommand(self.fo)
        return self
    
    def fanOn(self):
        gcodeCommand('M','106').writeCommand(self.fo)
        return self
    
    def homeAllAxis(self):
        gcodeCommand('G', '28').writeCommand(self.fo)
        return self
    
    def homeAxis(self, axisName):
        gcodeCommandCollection([gcodeCommand('G', '28'), gcodeCommand(axisName, '0')]).writeCommand(self.fo)
        return self
    
    def move(self, coordinates):
        commandList = [gcodeCommand('G','1')]
        for key, value in coordinates.items():
            commandList.append(gcodeCommand(key, round(value, 5)))
        gcodeCommandCollection(commandList).writeCommand(self.fo)
        return self
    
    def resetExtruder(self):
        self.setPosition({'E': 0})
        return self
        
    def retractedMove(self, coordinates, retraction, localE):
        self.move({ 'E': localE - retraction['length'] })
        self.move(coordinates)
        self.move({ 'E': localE })
    
    def setPosition(self, coordinates):
        commandList = [gcodeCommand('G','92')]
        for key, value in coordinates.items():
            commandList.append(gcodeCommand(key, value))
        gcodeCommandCollection(commandList).writeCommand(self.fo)
        return self
    
    def setTemperature(self, temperature):
        gcodeCommandCollection([gcodeCommand('M', '104'), gcodeCommand('S', temperature)]).writeCommand(self.fo)
        return self
    
    def setUnitsToMillimeters(self):
        gcodeCommand('G', '21').writeCommand(self.fo)
        return self
  
    def solenoidOpen(self):
        gcodeCommand('M', '106').writeCommand(self.fo)
    
    def solenoidClose(self):
        gcodeCommand('M', '107').writeCommand(self.fo)
  
    def useAbsoluteCoordinates(self):
        gcodeCommand('G', '90').writeCommand(self.fo) 
        return self
   
    def useAbsoluteDistances(self):
        gcodeCommand('M', '82').writeCommand(self.fo)
        return self
    
           
#gcode command class   
class gcodeCommand:
    def __init__(self, commandLetter, commandNumber):
        self.commandLetter = commandLetter
        self.commandNumber = commandNumber
        
    def getString(self):
        return self.commandLetter + str(self.commandNumber)

    def writeCommand(self, fo):
        fo.write(self.getString() + '\n')
        return fo


#gcode command inherited classes
class gcodeCommandCollection(gcodeCommand):
    def __init__(self, commandList):
        self.commandList = commandList
    
    def getString(self):
        string = ''
        for command in self.commandList:
            if (string):
                string += ' '
            string += command.getString()
        return string


class gcodeComment(gcodeCommand):
    def __init__(self, comment):
        self.comment = comment
        
    def getString(self):
        return ';' + self.comment     
    